#include <LEDA/basic.h>
#include <limits.h>
#include "evenDegree.h"


#ifndef INFINITY
#define INFINITY INT_MAX/2
#endif


int buildCostMatrix(const MixedGraph<NodeInfo,EdgeInfo>& graph,
                             SymMatrix<int>& costMatrix) {
  
   /* Initialization of the cost matrix:
      The distance between two nodes is set to infinity, if they
      are different, and to zero otherwise. 
   */
   costMatrix.init(INFINITY);
   costMatrix.setDiag(0);
  

   /* Internal integer numbering of the nodes of graph:
      If v is the n-th node of graph, then intNodes[v]=n-1.
      ( 1<=n<=graph.numberOfNodes() )
   */ 
   node_array<int> intNodes(graph.getDiGraph());
   NodeIt nIt(graph.getDiGraph());
   int count = 0;
 
   while (nIt.valid()) 
      intNodes[(nIt++).get_node()] = count++;


   /* Filling the cost matrix:
      Iterate over all arcs and edges of graph and insert their
      cost into the cost matrix, using the integer numbering
      of the nodes of graph.
   */
   edge a;
   forall_edges(a, graph.getDiGraph()) {   // arcs
      int intSource = intNodes[graph.source(a)];
      int intTarget = intNodes[graph.target(a)];
      costMatrix.at(intSource,intTarget) = graph[a].cost;
   }

   edge e;
   forall_edges(e, graph.getUGraph()) {   // edges
      int intNode1 = intNodes[graph.source(e)];
      int intNode2 = intNodes[graph.target(e)];
      costMatrix.at(intNode1,intNode2) = graph[e].cost;
   }


   return 0;
}
   


int floyd(const SymMatrix<int>& costMatrix, SymMatrix<int>&
		 matrixA, SymMatrix<int>& matrixP) {
   int n = costMatrix.getDim();

   matrixA = costMatrix;
   matrixP.init(INFINITY);

   for (int i=0; i<n; i++) {
     for (int j=0; j<n-1; j++) {
       for (int k=j+1; k<n; k++) {
	 if (matrixA.at(j,i) + matrixA.at(i,k) < matrixA.at(j,k)) { 
           matrixA.at(j,k) = matrixA.at(j,i)+matrixA.at(i,k);
           matrixP.at(j,k) = i;
         }
       }/*for k*/
     }/*for j*/
   }/*for i*/

   return 0;
}           



int evenDegree(MixedGraph<NodeInfo,EdgeInfo>& graph) {
 
   const int n = graph.numberOfNodes();

   
   /* Internal integer numbering of the nodes of graph:
      If v is the n-th node of graph, then intNodes[v]=n-1
      and graphNodes[n-1]=v. ( 1<=n<=graph.numberOfNodes() ) 
   */ 

   NodeIt nIt(graph.getDiGraph());     
   node_array<int> intNodes(graph.getDiGraph());
   int count = 0;
   while(nIt.valid()) 
      intNodes[(nIt++).get_node()] = count++;

   array<node> graphNodes(n);
   nIt.reset();   
   for (int i=0; i<n; i++,nIt++)
      graphNodes[i] = nIt.get_node();   
   


   /* The FIRST STEP of algorithm EVENDEGREE:
      Identify the nodes of graph of odd degree.

      With these odd nodes, a new parameterized undirected graph matchingGraph
      is build such that every node v of matchingGraph corresponds to the odd
      node matchingGraph[v] of graph.
      The edges that will be inserted into matchingGraph will have integer
      weights (see below). 
   */

   UGRAPH<node,int> matchingGraph;

   #ifdef PRINTLEVEL
      cout << "  Identifying the odd nodes..." << endl;
   #endif

   count=0;
   node nd;
   forall_nodes(nd, graph.getDiGraph()) {
      if (graph[nd].degree%2 == 1) {  
         matchingGraph.new_node(nd);
         count++;
      }
   } 

   #ifdef PRINTLEVEL
      cout << "  ...found " << count << " odd nodes." << endl;
   #endif


   // If there isn`t any odd node, nothing is left to do.
   if (!count)
      return 0;


   /* The SECOND STEP of algorithm EVENDEGREE:
      Find all shortest paths between the odd nodes of graph, ignoring
      arc direction.
 
      This is done like this:

        1. Build the cost matrix of graph. This ignores arc direction.          
   */
 
   #ifdef PRINTLEVEL
      cout << "  Building the cost matrix of the graph..." << endl;
   #endif

   SymMatrix<int> costMatrix(n);
   buildCostMatrix(graph,costMatrix);


   /*   2. Perform Floyd`s algorithm to find all shortest path
           distances. (The shortest paths themselves are also
           found but are not needed for the moment.) 
           Find the maximum "maxDistance" of these distances.
   */

   #ifdef PRINTLEVEL
      cout << "  Performing algorithm FLOYD..." << endl;
   #endif

   SymMatrix<int> matrixA(n);
   SymMatrix<int> matrixP(n);
   int maxDistance = 0;
   
   floyd(costMatrix,matrixA,matrixP);

   for (int i=0; i<n; i++)
     for (int j=i+1; j<n; j++) 
         if (matrixA.at(i,j)>maxDistance)
           maxDistance=matrixA.at(i,j);

   if (maxDistance == INFINITY) {
     cerr << "ERROR: Input graph was not connected !";
     return 1;
   }
    
   

   /* The THIRD STEP of algorithm EVENDEGREE:
      Perform a minimum-cost matching of the odd nodes of graph
      using the shortest path distances.  

      This is done like this:

        1. Make matchingGraph a complete graph. The weight of every 
           inserted edge (v,w) of matchingGraph is set to:
                       maxDistance+1 - dist,
           where dist is the shortest path distance between matchingGraph[v]
           and matchingGraph[w]).

           This is done to transform the MINIMUM-COST matching problem
           into a MAXIMUM-WEIGHT matching problem, which can be solved
           by the LEDA function MAX_WEIGHT_MATCHING(..).           
   */
   
     
   NodeIt it1(matchingGraph);
   NodeIt it2(matchingGraph);
   
   while(it1.valid()) {  
      node node1 = it1.get_node();  //Iterate over all nodes of matchingGraph.
      
      it2.update(matchingGraph.succ_node(node1));     // Iterate over all... 
      while(it2.valid()) {                // ...nodes of matchingGraph... 
        node node2 = it2.get_node();      // ...stored after node1.

        node graphNode1 = matchingGraph[node1];
        node graphNode2 = matchingGraph[node2];
        int intNode1 = intNodes[graphNode1];
        int intNode2 = intNodes[graphNode2];

        int weight = maxDistance+1 - matrixA.at(intNode1,intNode2);
 
        matchingGraph.new_edge(node1,node2,weight);
     
        it2++;
      }       
      it1++;
   }
   
   // collect all weights in an edge_array<int>
   edge_array<int> weights = matchingGraph.edge_data();


   /*   2. Perform a maximum weight matching of all nodes of matchingGraph.
           MAX_WEIGHT_MATCHING returns the matched edges as a
           list<edge>. 
   */

   #ifdef PRINTLEVEL
      cout << "  Performing a maximum weight matching..." << endl;
   #endif

   list<edge> matching = MAX_WEIGHT_MATCHING(matchingGraph,weights);



   /* The FOURTH STEP of algorithm EVENDEGREE:
      For each edge (v,w) of matchingGraph used in the matching:
      Traverse the shortest path between matchingGraph[v] and
      matchingGraph[w] in graph.
      Whenever an arc is passed, insert an additional copy of it
      into graph.
      Whenever an edge is passed, insert an additional copy of it
      into graph.
   */

   #ifdef PRINTLEVEL
      cout << "  Traversing the shortest paths between" 
           << endl << "  the odd nodes of the input graph..." << endl;
   #endif

   edge e;
   forall(e,matching) {

     // Use the integer numbering of the nodes of graph (see top):
      int from = intNodes[matchingGraph[matchingGraph.source(e)]]; 
      int to = intNodes[matchingGraph[matchingGraph.target(e)]]; 

      insertShortestPath(graph,from,to,matrixP,graphNodes);
   }
   
   return 0;  
}




int insertShortestPath(MixedGraph<NodeInfo,EdgeInfo>& graph, int from,
    int to, const SymMatrix<int>& matrixP, const array<node>& graphNodes) {
 
   // Translate "integer nodes" into nodes of graph.  
   node node1=graphNodes[from];
   node node2=graphNodes[to];


   int via=matrixP.at(from,to);

   if (via == INFINITY) {   
      /* The shortest path between node1 and node2 is the edge/arc
         [node1,node2].
      */
      
      edge e = graph.findEdge(node1, node2);

      if (e != nil) {  // There is an edge (node1,node2).
         
         EdgeInfo info;
         info.multi = 1;
 
         // Insert one additional copy of edge (node1,node2) into graph.
         graph.insertEdge(node1, node2, info);
      }
      else {  // There must be an arc <node1,node2> or <node2,node1>.
 
	 EdgeInfo info; 
         info.multi = 1;

         // Determine the direction of the arc and insert one
         // additional copy:
         if (graph.findArc(node1,node2) != nil)
	    graph.insertArc(node1,node2,info);
         else   
	    graph.insertArc(node2,node1,info); 
	 
      }
   }
   else { 
      /* The shortest path between node1 and node2 visites the node 
         graphNodes[via] ==> two recursive calls of this function.
      */

      insertShortestPath(graph,from,via,matrixP,graphNodes);
      insertShortestPath(graph,via,to,matrixP,graphNodes);
   }

   return 0;
}



