/** \file evenDegree.h 
 * Exported functions: evenDegree().
*/
#ifndef EVENDEGREE_H
#define EVENDEGREE_H


#include <LEDA/array.h>
#include "mixedGraph.h"
#include "symMatrix.h"



/** Builds the cost matrix of graph ignoring arc direction.
 * \param graph A constant reference to a mixed graph of type MixedGraph<NodeInfo,EdgeInfo>.
 * \param costMat A reference to a symmetric matrix of dimension 
 * |V| and type SymMatrix<int>. Afterwards, it will contain the cost
 *  matrix of the graph:
 *   costMat.at(n,m) = distance between the (n+1)-th and the (m+1)-th
 *   node of graph. (0 <= n,m <= |V|-1)     
 *
 * \return 0, if it worked
*/
int buildCostMatrix(const MixedGraph<NodeInfo,EdgeInfo>& graph,
                             SymMatrix<int>& costMat);




/** An implementation of Floyd`s algorithm to find all pairs shortest paths
 * and their distances. It will take time O(\f$|V|^3\f$).
 *
 * \param costMat A constant reference to the cost matrix of a graph,
 * such as created by function ::buildCostMatrix.
 * Warning: ALL ENTRIES OF COSTMAT MUST BE NONNEGATIVE.
 * \param matrixA A reference to a symmetric matrix with dimension |V|.
 * \param matrixP A reference to a symmetric matrix with dimension |V|.
 *
 * Afterwards, for every element (n,m) (n and m are ints) of matrixA will hold
 * the following (Let v be the (n+1)-th node and w be the (m+1)-th node of graph):
 *
 * matrixA.at(n,m) = distance of the shortest path between node v and node w
 *
 * matrixP.at(n,m) = \f$\infty\f$, if the shortest path between v and w is the arc or
 * edge [v,w], and  = via if the (via+1)-th node of graph is visited on the shortest
 * path. This matrix is used to recursively find the shortest paths, see 
 * ::insert_shortest_path.
 *
*/
int floyd(const SymMatrix<int>& costMat, SymMatrix<int>& matrixA,
		 SymMatrix<int>& matrixP);




/** Makes the degree of each node even.
 *  This function takes a connected mixed graph and adds some arcs and
 *  edges to make the degree of each node even while minimizing the cost
 *  of the added arcs/edges.
 * \param graph A reference to a mixed graph of type MixedGraph<NodeInfo,EdgeInfo>.
 * \return 0, if it worked.
 * \return 1, otherwise (for example, if graph was not connected).
*/
int evenDegree(MixedGraph<NodeInfo,EdgeInfo>& graph);





/** Traverses the shortest path between two nodes and inserts copies
 *  of the visited arcs/edges.

 *  \a from and \a to are integers (0 <= from,to <= |V|-1). 
 *  Let v be the (from+1)-th node and w be the (to+1)-th node of graph.
 *  Then this function does the following: The shortest path between
 *  v and w is traversed. Whenever an arc is traversed, it will be
 *  duplicated and a corresponding statement is true for edges.

 *  \param graph A reference to a mixed graph of type MixedGraph<NodeInfo,EdgeInfo>.
 *  \param from An integer which refers to an integer numbering of the
 *  nodes of graph. It represents the (from+1)-th node of graph.
 *  \param to An integer which refers to an integer numbering of the
 *  nodes of graph (see below: graphNodes). It represents the (to+1)-th node of graph.
 *  \param matrixP A constant reference to the symmetric matrix matrixP which is 
 *  produced by function ::floyd.
 *  \param graphNodes A constant reference to an array of nodes of graph with the
 *  following property: graphNodes[n] = (n+1)-th node of graph.
 *  \return 0, if it worked.
*/
int insertShortestPath(MixedGraph<NodeInfo,EdgeInfo>& graph , int from,
	      int to, const SymMatrix<int>& matrixP, const array<node>& graphNodes);





#endif
