#include "mixedGraph.h"
#include "inOutDegree.h"
#include <LEDA/basic.h>

#ifndef INFINITY
#include <limits.h>
#define INFINITY INT_MAX/2
#endif



static void removeZeroReversals(GRAPH<node,edge>& diGraphCopy, edge arc,
			   edge_array<int>& capacity, edge_array<int>& flow);


  
int inOutDegree(MixedGraph<NodeInfo,EdgeInfo>& graph, GRAPH<node,int>& extraCopies) {
   
   /* Make a copy diGraphCopy of the directed subgraph diGraph of graph
      (see documentation of the class MixedGraph<vtype,etype> ) such that
      every node v of diGraphCopy corresponds to the node diGraphCopy[v]
      of graph and every arc a of diGraphCopy corresponds to the arc
      diGraphCopy[a] of graph.

      In addition to that, a node_array<node> diGraphCopyNodes is created
      such that every node w of graph corresponds to the node
      diGraphCopyNodes[w] of diGraphCopy. 
   */
      
   #ifdef PRINTLEVEL
      cout << "  Copying the arcs of the graph..." << endl;
   #endif

   GRAPH<node,edge> diGraphCopy;
   CopyGraph(diGraphCopy,graph.getDiGraph());     

   node_array<node> diGraphCopyNodes(graph.getDiGraph());
   NodeIt diIt(graph.getDiGraph());
   NodeIt mIt(diGraphCopy);
   for(; diIt.valid(); diIt++,mIt++)
      diGraphCopyNodes[diIt.get_node()] = mIt.get_node();
   
   
   
   /* Introduce the edge_arrays capacity and cost. They are valid for
      all arcs in diGraphCopy and for 4*graph.numberOfEdges() 
      additional arcs, as for each edge in graph four "oriented edges" (=arcs)
      will be inserted into diGraphCopy (see below).
   */

   edge_array<int> capacity(diGraphCopy,4*graph.numberOfEdges()+graph.numberOfArcs(),0);
   edge_array<int> cost(diGraphCopy,4*graph.numberOfEdges()+graph.numberOfArcs(),0);



   /* For all arcs of diGraphCopy, set the upper capacity for the following
      network flow algorithm to infinity, and the cost to the cost
      of the arcs.
   */

   edge a;
   forall_edges(a,diGraphCopy) {
      capacity[a] = INFINITY;
      cost[a] = graph[diGraphCopy[a]].cost;
   }


   /* For all edges (v,w) of diGraphCopy:

      Insert two oriented edges <v,w> and <w,v> with upper capacity
      equal to the multiplicity of (diGraphCopy[v],diGraphCopy[w]) and
      zero cost into diGraphCopy.
      (==> Each edge can be oriented once without cost!)

      Insert two oriented edges <v,w> and <w,v> with upper capacity
      equal to infinity and cost equal to the cost of
      (diGraphCopy[v],diGraphCopy[w]) into diGraphCopy.
      (==> Additional copies of oriented edges can be made without
      capacity constraints but with cost!)

      For each oriented edge o, diGraphCopy[o] is set to nil.
      As diGraphCopy[a] != nil for all arcs that have not been edges before,
      the oriented edges can always be identified as such.
   */
     
   #ifdef PRINTLEVEL
      cout << "  Inserting 4 oriented edges for each edge..." << endl;
   #endif
 
   edge e;
   forall_edges(e,graph.getUGraph()) {
      
      // v and w are nodes of diGraphCopy !
      node v = diGraphCopyNodes[graph.source(e)];
      node w = diGraphCopyNodes[graph.target(e)];

      const int multi = graph[e].multi;
      const int costs = graph[e].cost;

      edge edge1 = diGraphCopy.new_edge(v,w); 
      diGraphCopy.assign(edge1,nil);
      capacity[edge1] = multi;
      cost[edge1] = 0;
      
      edge edge2 = diGraphCopy.new_edge(w,v);
      diGraphCopy.assign(edge2,nil);
      capacity[edge2] = multi;
      cost[edge2] = 0;

      edge edge3 = diGraphCopy.new_edge(v,w);
      diGraphCopy.assign(edge3,nil);
      capacity[edge3] = INFINITY;
      cost[edge3] = costs;

      edge edge4 = diGraphCopy.new_edge(w,v);
      diGraphCopy.assign(edge4,nil);
      capacity[edge4] = INFINITY;
      cost[edge4] = costs;
   }
  
   
   /* Perform a minimum-cost network flow algorithm on diGraphCopy, where
       - the lower capacity is zero for each arc/oriented edge in diGraphCopy,
       - the upper capacities and the costs are already set (see above),
       - the supply (demand resp., if it is negative) of node n of graph
         is set to indegree(n)-outdegree(n), 
      to make indegree(n)=outdegree(n) while minimizing the cost
      of the additional arcs/oriented edges.
   */   

   #ifdef PRINTLEVEL
      cout << "  Performing a minimum-cost network flow algorithm " 
           << endl << "to make indegree equal to outdegree in each node..."
           << endl;
   #endif
 
   node_array<int> supply(diGraphCopy);
   edge_array<int> flow(diGraphCopy);   

   node nd;
   forall_nodes(nd,diGraphCopy) {
      int indegree = graph[diGraphCopy[nd]].indegree;
      int outdegree = graph[diGraphCopy[nd]].outdegree;
  
      supply[nd] =  indegree - outdegree; 
   }

   if (!MIN_COST_FLOW(diGraphCopy,capacity,cost,supply,flow)) {
      cout << "ERROR: Min.-cost flow algorithm failed!" << endl;
      return 1;
   }


   /* Add graph.numberOfNodes() nodes to the (empty) graph extraCopies such
      that every node v of extraCopies corresponds to the node
      extraCopies[v] of graph.
      Create the node_array<node> such that every node w of graph 
      corresponds to the node extraCopiesNodes[w] of extraCopies.
      (one-to-one mapping of the nodes of graph and extraCopies)
   */
   
   node_array<node> extraCopiesNodes(graph.getDiGraph());
   NodeIt nIt(graph.getDiGraph());

   while(nIt.valid()) {
      extraCopiesNodes[nIt.get_node()] = extraCopies.new_node(nIt.get_node()); 
      nIt++;
   }
    


   /* Use the result of the flow algorithm to make indegree equal
      to outdegree in each node of graph:
   */

   edge arc;
   forall_edges(arc,diGraphCopy) {  //Iterate over all arcs of diGraphCopy

     EdgeInfo arcInfo; // will contain the information of arcs inserted into graph

     /* It happens that algorithm MIN_COST_FLOW orients
        ONE edge TWICE in spite of the capacity constraints:
        To see this, let (n1,n2) be an edge of graph with multi=1. 
        Then diGraphCopy will contain the oriented edges <v,w> and <w,v>
        (where v/w:=diGraphCopyNodes[n1/n2]) with zero cost and capacity 1. 
        As they cost nothing, MIN_COST_FLOW could set both,
        flow[<v,w>] AND flow[<w,v>], to 1 (and this really happens!).
        Of course, one can only orient edges of graph that exist,
        and this is why we have to call the following function:
     */    
     if (capacity[arc]!=INFINITY && flow[arc]) // situation mentioned above
        removeZeroReversals(diGraphCopy,arc,capacity,flow);


     if (flow[arc]) { 

       // source and target are nodes of graph !
       node source = diGraphCopy[diGraphCopy.source(arc)]; 
       node target = diGraphCopy[diGraphCopy.target(arc)]; 

       if (capacity[arc]!=INFINITY) {  
       // arc = former edge, which has been oriented without cost  
         
	  // Find the corresponding edge of graph: 
	  edge e = graph.findEdge(source,target);  
          arcInfo.cost = graph[e].cost;

          // Remove flow[arc] occurences of e from graph:
          EdgeInfo eInfo = graph[e]; 
          eInfo.multi -= flow[arc];
          graph.assign(e,eInfo);

          // last occurence of e removed? 
          if (!eInfo.multi)
             graph.delEdge(e);    
          
       }
       else {  // arc = additional copy of an arc or an oriented edge

	  /* Insert the arc into the graph extraCopies of extra copies.
            Note that the incident nodes have to be translated       
            using "extraCopiesNodes" to really access the nodes of extraCopies
	  */
          node extraCopiesSource = extraCopiesNodes[source];
          node extraCopiesTarget = extraCopiesNodes[target];
          extraCopies.new_edge(extraCopiesSource,extraCopiesTarget,flow[arc]);

          arcInfo.cost=cost[arc];
       }

            
       arcInfo.multi = flow[arc];
       graph.insertArc(source,target,arcInfo);    
      
     }
   }
  
   return 0;
} 



static void removeZeroReversals(GRAPH<node,edge>& diGraphCopy, edge arc,
			     edge_array<int>& capacity, edge_array<int>& flow) {
 
   // Searching for the arc <diGraphCopy.target(arc),diGraphCopy.source(arc)> 
   node source = diGraphCopy.target(arc);
   node target = diGraphCopy.source(arc);

   InAdjIt inIt(diGraphCopy,target);
   OutAdjIt outIt(diGraphCopy,source);

   bool found1 = false;
   bool found2 = false;
   while (1) {  
      /* Iterate over the arcs starting in source and over the arcs ending 
         in target until the arc <source,target> is found.
      */

      found1 = (inIt.valid() && diGraphCopy.source(inIt.get_edge()) == source 
                            && capacity[inIt.get_edge()] != INFINITY);
      found2 = (outIt.valid() &&  diGraphCopy.target(outIt.get_edge()) == target
                            && capacity[outIt.get_edge()] != INFINITY);
      if (found1 || found2)
         break;

      if (inIt.valid()) inIt++;
      if (outIt.valid()) outIt++;
   }

 
   edge reverseArc; 
   if (found1)
      reverseArc = inIt.get_edge();
   else
      reverseArc = outIt.get_edge();

   /* Subtract min(flow[<source,target>],flow[<target,source>]) from
      the two flow values. Afterwards, only one of the two values
      can be positive ==> This is the wanted.
   */
   int tooMuch = flow[arc]<flow[reverseArc] ? flow[arc] : flow[reverseArc];      
   flow[arc] -= tooMuch;
   flow[reverseArc] -= tooMuch;
}
   

