

#ifndef MIXEDGRAPH_H
#define MIXEDGRAPH_H


#include <LEDA/graph.h>
#include <LEDA/ugraph.h>
#include <LEDA/graph_iterator.h>
#include <LEDA/edge_array.h>
#include <LEDA/graph_alg.h>
#include <LEDA/list.h>



/** A structure to store additional arc/edge data. 
 * The class MixedGraph<VType,EType> is a class template for parameterized
 * mixed graphs. So one have to decide which data type is used to store
 * the additional arc/edge data. (==> The template parameter \a EType 
 * has to be specialized.)
 * The structure \a EdgeInfo is such a data type.
*/ 
struct EdgeInfo {
   int cost;  ///< The cost of an arc/edge.

   int multi;  ///< The multiplicity of an arc/edge.

   /** The overloaded input operator >>.
    * In case that the data type used to represent the arc/edge
    * data is not a basic data type (like int, float, ...), one
    * have to overload the input operator >> and to specify how to
    * read in the information of an arc/edge.
    * Here, the variables \a cost and \a multi are read sequentially.
   */   
   friend istream& operator >> (istream& is, EdgeInfo& info) {
     is >> info.cost >> info.multi;
     return is;
   }

   /** The overloaded output operator <<.
    * In case that the data type used to represent the arc/edge
    * data is not a basic data type (like int, float, ...), one
    * have to overload the output operator << and to specify the
    * output generation of the arc/edge information.
    * Here, the following output is generated:
    *    c: (the value of variable \a cost) m: (the value of variable \a multi)
   */
   friend ostream& operator << (ostream& os, const EdgeInfo& info) {
     os << "c:" << info.cost << " m:" << info.multi;
     return os;
   }    
};


/** A structure used to store additional node data. 
 * The class MixedGraph<VType,EType> is a class template for parameterized
 * mixed graphs. So one have to decide which data type is used to store
 * the additional node data. (==> The template parameter \a VType 
 * has to be specialized.)
 * The structure \a NodeInfo is such a data type.
*/
struct NodeInfo {
   int indegree;  ///< The number of arcs ending in the node. 

   int outdegree;  ///< The number of arcs starting in the node.

   int degree;  ///< The number of arcs/edges incident with the node.

    /** The overloaded input operator >>.
    * In case that the data type used to represent the node
    * data is not a basic data type (like int, float, ...), one
    * have to overload the input operator >> and to specify how to
    * read in the information of a node.
    * Here, the variables \a indegree, \a outdegree and \a degree are read
    * sequentially. 
   */
   friend istream& operator >> (istream& is, NodeInfo& info) {
      is >> info.indegree >> info.outdegree >> info.degree;
      return is; 
   }

   /** The overloaded output operator <<.
    * In case that the data type used to represent the node
    * data is not a basic data type (like int, float, ...), one
    * have to overload the output operator << and to specify the
    * output generation of a node information.
    * Here, the following output is generated:
    *    i: (the value of variable \a indegree ) o: (the value of variable 
    * \a outdegree ) d: (the value of variable \a degree )
   */
   friend ostream& operator << (ostream& os, const NodeInfo& info) {
     os << "i:" << info.indegree << " o:" << info.outdegree
        << " d:" << info.degree;
     return os;
   }     
};




/** A class template for parameterized mixed graphs.
 * An instance of the data type MixedGraph<VType,EType> consists of a list
 * V of \a nodes, a list A of (directed) \a arcs and a list E of (undirected)
 * \a edges. 
 * Every node contains an element of a data type \a VType and every
 * arc or edge contains an element of a data type \a EType.
 *
 * Nodes have data type \a node.
 * Both, arcs \a and edges, have data type \a edge! 
 * 
 * \a node and \a edge are LEDA item types. (see the LEDA User Manual
 * for more information)
 * This is why every existing node, arc or edge belongs to \a exactly
 * \a one MixedGraph! ==> Distinct MixedGraphs have disjoint node,arc and edge
 *  lists.
 * 
 * Internally, a mixed graph of type MixedGraph is represented by two (private)
 * graphs \a diGraph and \a uGraph:
 * The directed parametrized graph \a diGraph of LEDA type GRAPH<VType,EType>
 * contains all arcs and the undirected parameterized graph \a uGraph of LEDA type
 * UGRAPH<VType,EType> contains all edges of G.
 * As two graphs cannot have the same node list (see above), there
 * are also two internal disjoint node lists, say \f$V_{diGraph}\f$ and 
 * \f$V_{uGraph}\f$.  
 * As, of course, one graph should have only one node list, we have made
 * the following decision: \f$V=V_{diGraph}\f$
 * 
*/
template <class VType, class EType>
class MixedGraph {
   public:

      MixedGraph(int n);  ///< A constructor for a graph with n nodes. 
      MixedGraph(int n, VType startInfo);  /**< A constructor for a graph
      with n nodes. The additional data of all nodes is set to \a startInfo. */

      node source(edge e) const; ///< Returns the source node of arc/edge e. 
      node target(edge e) const; ///< Returns the target node of arc/edge e.

      int numberOfNodes() const; ///< Returns the number of nodes in the graph.
      int numberOfArcs() const;  ///< Returns the number of arcs in the graph.
      int numberOfEdges() const; ///< Returns the number of edges in the graph.

      list<node> allNodes() const; ///< Returns the list of all nodes in the graph.
      list<edge> allEdges() const; ///< Returns the list of all edges in the graph. 
      list<edge> allArcs() const; ///< Returns the list of all arcs in the graph.
 
      node firstNode() const; ///< Returns the first node stored in the graph.
      node succNode(node v) const; ///< Returns the successor of node v.
      node predNode(node v) const; ///< Returns the predecessor of node v.
      node lastNode() const;  ///< Returns the last node stored in the graph.
      edge firstArc() const;  ///< Returns the first arc stored in the graph.
      edge succArc(edge a) const; ///< Returns the successor of arc a.
      edge predArc(edge a) const; ///< Returns the predecessor of arc a.
      edge lastArc() const; ///< Returns the last arc stored in the graph.
      edge firstEdge() const; ///< Returns the first edge stored in the graph.
      edge succEdge(edge e) const; ///< Returns the successor of edge e.
      edge predEdge(edge e) const; ///< Returns the predecessor of edge e.
      edge lastEdge() const;  ///< Returns the last edge stored in the graph. 
 
      edge firstAdjArc(node v) const; ///< Returns the first arc starting in v.
      edge firstInArc(node v) const; ///< Returns the first arc ending in v.
      edge firstAdjEdge(node v) const; ///< Returns the first edge adjacent to v.
      edge adjSucc(edge e) const; /**< Returns the next arc (edge) starting in
        (adjacent to) source(e). */

      edge findArc(node s, node t) const; /**< Returns arc <s,t> if it
         exists and nil otherwise. */
      edge insertArc(node s, node t); ///< Adds an arc <s,t> to the graph. 
      edge insertArc(node s, node t, EType info); /**< Adds an arc <s,t> and
         sets the additional arc data to info. */
      void delArc(edge a); ///< Deletes arc a completely from the graph.
     
      edge findEdge(node v, node w) const; /**< Returns edge (v,w) if it
         exists and nil otherwise. */
      edge insertEdge(node v, node w); ///< Adds an edge (v,w) to the graph.
      edge insertEdge(node v, node w, EType info); /**< Adds an edge (v,w) and sets the additional edge data to info */
      void delEdge(edge e); ///< Deletes edge e completely from the graph.

      /// Prints node v on the output stream O. 
      void printNode(node v, ostream& O = cout) const;

      /** Prints arc a on the output stream O. The arc is represented
       * by an arrow pointing from source to target.
      */
      void printArc(edge a, ostream& O = cout) const; 

      /** Prints edge e on the output stream O. The edge is represented
       * by an undirected line between its endpoints.
      */
      void printEdge(edge e, ostream& O = cout) const;

      /** Prints the graph on the output stream O. */
      void print(ostream& O = cout) const;

      VType inf(node v) const; ///< Returns the information of node v.

      /** Returns a constant reference to the information of node v. */
      const VType& operator[](node) const; 

      EType inf(edge e) const; ///< Returns the information of arc/edge e.

      /** Returns a constant reference to the information of arc/edge e. */
      const EType& operator[](edge) const;

      void assign(node v, VType x); ///< Makes x the information of node v.
      void assign(edge e, EType x); ///< Makes x the information of arc/edge e

      /** Returns true iff v is a node of the graph.*/ 
      bool isNode(node v) const; 

      /** Returns true iff a is an arc of the graph.*/
      bool isArc(edge a) const;

      /** Returns true iff e is an edge of the graph.*/
      bool isEdge(edge e) const;
     
      /** Returns a constant reference to the private member variable
       * GRAPH<VType,EType> \a diGraph. */
      const GRAPH<VType,EType>& getDiGraph() const;

      /** Returns a constant reference to the private member variable
       * UGRAPH<VType,EType> \a uGraph. */
      const UGRAPH<VType,EType>& getUGraph() const;

     
   private:
      int nodeCount; ///< The number of nodes in the graph.

      /** The private directed graph containing the arcs of the graph.
      */
      GRAPH<VType,EType> diGraph;

      /** The private undirected graph containing the edges.
      */
      UGRAPH<VType,EType> uGraph;

      /** A private node_array to deal with  \f$V_{diGraph}\f$ and
       * \f$V_{uGraph}\f$.  
       * As there exist two disjoint node lists,  \f$V_{diGraph}\f$ and
       * \f$V_{uGraph}\f$, there must be a one-to-one mapping of the
       * elements of these lists:
       * 
       * The node of \a uGraph, which corresponds to the node \f$v_{diGraph}\f$
       * of \a diGraph is uNodes[\f$v_{diGraph}\f$].   
      */
      node_array<node> uNodes;

      /** A private node_array used to deal with  \f$V_{diGraph}\f$ and
       * \f$V_{uGraph}\f$.  
       * As there exist two disjoint node lists,  \f$V_{diGraph}\f$ and
       * \f$V_{uGraph}\f$, there must be a one-to-one mapping of the
       * elements of these lists:
       * 
       * The node of \a diGraph, which corresponds to the node \f$v_{uGraph}\f$
       * of \auGraph is diNodes[\f$v_{uGraph}\f$].   
      */
      node_array<node> diNodes; 
};



#include "mixedGraph.cpp"


#endif





