

/**
 * \mainpage Overview
 * \section section1 Abstract
 * This is an C++ implementation of the 1973 Edmonds and Johnson Mixed
 * Chinese Postman algorithm. This is an approximation algorithm for the
 * Mixed Chinese Postman Problem which has been shown to be NP-complete.
 * The main data structure of this implementation is the class template
 * MixedGraph<VType,EType>, which derives from data structures for directed 
 * and undirected graphs taken from the Library for Efficient Data structures
 * and Algorithms (LEDA).
 * 
 * \section section2 The Edmonds and Johnson Algorithm

 * \subsection subsection1 The Mixed Chinese Postman Problem
 * This problem requires that all (undirected) edges and (directed) arcs
 * of a mixed graph be included at least once in a tour of minimum cost,
 * where all arcs/edges of the graph contain nonnegative costs.
 * As this problem has been shown to be NP-complete, one have to use
 * approximation algorithms.

 * \subsection subsection2 The Edmonds And Johnson Algorithm
 * To understand the idea of this approximation algorithm, one have
 * to know about the following basic result:
 *
 * A tour using each edge and arc just once exists if the following 
 * properties are satisfied: 1) The degree of each node of the graph must
 * be even, and 2) In each node the indegree must equal the outdegree.
 *
 * The Edmonds and Johnson algorithm essentially consists of three steps:
 * (The details of the algorithm are described in \e Greg \e Frederickson,
 * \e Approximation \e Algorithms \e for \e Some \e Postman \e Problems,  
 * \e J.ACM \e 26,3 \e (July \e 1979), \e 538-544.)
 *
 * 1) Make the degree of each odd node even by adding some arcs/edges.
 * 
 * 2) Make indegree equal to outdegree in each node by adding some arcs
 *    and/or by orienting some edges. Of course, in this step the degree
 *    of some nodes can be changed from even to odd, which destroys the
 *    result of step 1)!
 * 
 * 3) Make the degree of each odd node even, \a while \a preserving
 *    \a the \a property \a that \a indegree=outdegree \a in \a each \a node.
 *   
 * \section section3 The Implementation
 * This implementation of the Edmonds and Johnson algorithm is written
 * in GNU C++. It uses the Library for Efficient Data structures and 
 * Algorithms (LEDA) and won`t compile without LEDA.
 *
 * As LEDA only provides data structures for directed and undirected 
 * graphs, a class for (parameterized) mixed graphs had to be written:
 * The result is the class template MixedGraph<VType,EType>. 
 *
 *  
*/

#ifndef SYMMATRIX_H
#define SYMMATRIX_H


/** A class template for symmetric two-dimensional arrays \f$\in T^{n \times n}\f$.
 * This data structure is used for the efficient storage of symmetric 
 * two-dimensional arrays \f$\in T^{n \times n}\f$ with index range
 * {0,1,...,n-1}.
 * It is a very simple class with self-explaining member functions.
 * 
*/
template<class T>
class SymMatrix {

   public:
      SymMatrix(int n);  ///< A general constructor for a symmetric \
                              matrix  \f$\in T^{n \times n}\f$.
      SymMatrix(const SymMatrix<T>&); ///< A copy constructor.
      ~SymMatrix(); ///< A destructor.
      int getDim() const; ///< Returns the dimension of the matrix.
      SymMatrix<T>& operator=(const SymMatrix<T>&); ///< Assignment operator.
      void init(const T& value); ///< Sets all entries to value.
      T& at(int i, int j); ///< Reference to the entry at position (i,j).
      const T& at(int i, int j) const; ///< The same for constant objects.
      void setDiag(const T& value); ///< Sets all diagonal entries to value.

   private:
      int dim;
      int size;
      T *start;  

      int indexShift(int,int) const;
};



#include "symMatrix.cpp"

#endif






